from typing import Any

from loguru import logger
from pandas import DataFrame
from sqlalchemy import create_engine as _create_engine
from sqlalchemy.engine import Connection
from sqlalchemy.sql import text


class Engine:
    def __init__(self, /, url: Any) -> None:
        self.__engine = _create_engine(url)

    def connect(self, /) -> Connection:
        """
        with engine.connect() as connection:
            with connection.begin():
                ...
        """
        return self.__engine.connect()

    def __execute(
        self,
        connection: Connection,
        /,
        sql: text,
        params: dict[str, Any],
        *,
        debug: bool = False,
    ) -> Any:
        result = connection.execute(sql, params)
        if debug:
            logger.debug(f"\n{sql}\n{params}\n{result}")
        return result

    def dql(
        self,
        connection: Connection,
        /,
        sql: text,
        params: dict[str, Any],
        *,
        debug: bool = False,
    ) -> DataFrame:
        """
        Data Query Language: SELECT
        """
        rows = self.__execute(connection, sql, params, debug=debug).all()
        df = DataFrame(rows)
        if debug:
            logger.debug(f"\n{df}")
        return df

    def dml(
        self,
        connection: Connection,
        /,
        sql: text,
        params: dict[str, Any],
        *,
        debug: bool = False,
    ) -> None:
        """
        Data Manipulation Language: INSERT, UPDATE, DELETE
        """
        self.__execute(connection, sql, params, debug=debug)
